import torch
import sigpy as sp

from typing import Optional, Tuple
from wearable_mri.utils import np_to_torch, torch_to_np

class wavelet:

    def __init__(self, im_size, wave_name: Optional[str] = 'db4', level: Optional[int] = None):

        self.W = sp.linop.Wavelet(im_size, 
                                  wave_name=wave_name,
                                  level=level)
        
    def forward(self, 
                img: torch.Tensor) -> Tuple[torch.Tensor, torch.Tensor]:
        """
        Forward wavelet transform
        """
        img_sp = torch_to_np(img)
        dev = sp.get_device(img_sp)
        with dev:
            Wx = self.W(img_sp)
        return np_to_torch(Wx)
    
    def adjoint(self,
                Wx: torch.Tensor) -> torch.Tensor:
        """
        Adjoint wavelet transform
        """
        Wx_sp = torch_to_np(Wx)
        dev = sp.get_device(Wx_sp)
        with dev:
            x = self.W.H(Wx_sp)
        return np_to_torch(x)
        