import numpy as np
from wearable_mri.utils import normalize

import matplotlib
matplotlib.use('WebAgg')
import matplotlib.pyplot as plt

from skimage.metrics import peak_signal_noise_ratio 

# Filenames
file_phantom = 'data/brain_phantom.npy'
files = [
    'data/recons/sweep/CG_2.000e-03_-1.000e+00_-1.000e+00_3.652e-04.npy',
    'data/recons/sweep/ADMM_tv_2.000e-03_1.000e-04_3.162e-05_-1.000e+00.npy',
    'data/recons/sweep/ADMM_wav_2.000e-03_1.000e-04_4.217e-05_-1.000e+00.npy',
    'data/recons/sweep/ADMM_dncnn_2.000e-03_2.371e-04_1.000e-02_-1.000e+00.npy',
]
titles = ['CG', 'TV', 'Wavelet', 'DnCNN']

# No Noise 
img_slc = (slice(2, -8), slice(22, -22))
phantom = np.load(file_phantom)[img_slc]
vmin = 0
vmax = np.median(np.abs(phantom)) + 3*np.std(np.abs(phantom))
M = 5
fig, axs = plt.subplots(nrows=2, ncols=4, figsize=(11.43,7))
for i in range(4):
    # Load recon
    recon = np.load(files[i])[img_slc]
    recon = normalize(recon, phantom, ofs=True, mag=False)

    # Plot recon
    axs[0, i].imshow(np.abs(recon), cmap='gray', vmin=vmin, vmax=vmax)
    axs[0, i].set_title(titles[i])
    axs[0, i].axis('off')

    # Plot diff
    axs[1, i].imshow(np.abs(recon - phantom), cmap='gray', vmin=vmin, vmax=vmax / M)
    axs[1, i].axis('off')

    # Compute NRMSE and PSNR
    nrmse = 100 * np.linalg.norm(phantom - recon) / np.linalg.norm(phantom)
    data_range = np.max(np.abs(phantom)) - np.min(np.abs(phantom))
    psnr = peak_signal_noise_ratio(np.abs(phantom), np.abs(recon), data_range=data_range)

    # Plot RMSE and PSNR as text on diff image
    axs[1, i].text(0.2, 0.83, f'nRMSE = {nrmse:.2f}%', transform=axs[1,i].transAxes,
                   color='white', fontsize=12, weight='bold')
    axs[1, i].text(0.2, 0.9, f'PSNR   = {psnr:.2f}dB', transform=axs[1,i].transAxes,
                   color='white', fontsize=12, weight='bold')
fig.tight_layout()
fig.subplots_adjust(hspace=0, wspace=0)
plt.show()